use axum::response::IntoResponse;
use axum::response::Json;
use axum::response::Response;
use axum::http::status::StatusCode;
use serde::Serialize;

use crate::mycelium::MycQuerySettings;
use crate::mycelium::MycFormat;

pub trait MycView<S,F>: Serialize + Sized
where S: MycQuerySettings<F>, F: MycFormat {

	/// Returns the template name that will be used to select
	/// the template file.
	/// 
	/// If the name is "404" for an html response the template
	/// file "404.html" will be used.
	///
	/// Also ends up as the `view` variable in the template.
	fn get_template_name(&self) -> String;

	//TODO: Add query settings to these

	/// Returns the reponse code for the view.
	///
	/// The numeric value will be useable as `http_status` in the template.
	fn get_status_code(&self, settings: &S) -> StatusCode;

	/// If this returns a String it will be used as the cookie header.
	///
	/// See: [axum: Constructing a Cookie](https://docs.rs/axum-extra/0.8.0/axum_extra/extract/cookie/struct.Cookie.html#constructing-a-cookie)
	fn get_cookie_header(&self, _settings: &S) -> Option<String> { None }

	/// Update non-API responses after they have been built.
	///
	/// Useful for setting extra headers. Does noting by default.
	fn update_response(&self, _response: &mut Response, _settings: &S) {}

	/// Return an API-Response
	///
	/// By default causes the view to Serialize itself
	/// to a json response using serde.
	///
	/// Response code and cookie headers are queried in
	/// advance and set on the reulting response if it has a status code of 200.
	/// Otherwise it is assumed that the response genrating logic
	/// alredy took care of that.
	fn get_api_response(self, _settings: &S) -> Response {
		Json(self).into_response()
	}
}
