use serde::{Deserialize,Serialize};

use std::sync::Arc;
use crate::mycelium::HtmlTextJsonFormat;
use crate::mycelium::MycQuerySettings;

/* Response format */

pub type ResponseFormat = HtmlTextJsonFormat;

/* Query and Template Settings */

#[derive(Deserialize, Serialize, Clone)]
pub struct QuerySettings {
	pub format: ResponseFormat,
	pub lang: String,
	pub available_dns_resolvers: Vec<Selectable>,
	pub dns_resolver_id: Arc<str>,
}

#[derive(Deserialize, Serialize, Clone)]
pub struct Selectable {
	pub id: Arc<str>,
	pub name: Arc<str>,
	pub weight: i32,
}

impl MycQuerySettings<ResponseFormat> for QuerySettings {
	
	fn initalize_template_context(&self, context: &mut tera::Context) {
		context.insert("language", &self.lang);
		context.insert("dns_resolvers", &self.available_dns_resolvers);
		context.insert("dns_resolver_id", &self.dns_resolver_id);
	}

	fn get_format(&self) -> ResponseFormat {
		self.format.clone()
	}

}
